#  ---------------------------------------------------------------------
#   General database implementation of ODL-Union (v1.0) - data provider
#    v1.1
#    January 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package ODL::DBUnion::UnionDP;


use POSIX;

use Pure::EZXML;
use Pure::X2D;
use XOAI::XOAISP;
use ODL::DBUnion::UnionDB;

use XOAI::XOAIDP;
use vars ('@ISA');
@ISA = ("XOAI::XOAIDP");


# constructor
sub new
{
   my ($classname, $configfile) = @_;
   
   my $self = $classname->SUPER::new ();

   # get configuration from file
   my $con = new Pure::X2D ($configfile);
   $self->{'database'} = $con->param ('database', '');
   $self->{'dbusername'} = $con->param ('dbusername', '');
   $self->{'dbpassword'} = $con->param ('dbpassword', '');
   $self->{'table'} = $con->param ('table', 'irdb');
   $self->{'repositoryName'} = $con->param ('repositoryName', 'ODL Union Engine');
   $self->{'adminEmail'} = $con->param ('adminEmail', "mailto:someone\@somewhere");
   $self->{'recordlimit'} = $con->param ('recordlimit', 100);
   
   # set internal variables
   $self->{'resumptionseparator'} = '!';

   # open database
   $self->{'dbh'} = ODL::DBUnion::UnionDB::openDatabase ($self->{'database'},
     $self->{'dbusername'}, $self->{'dbpassword'}, $self->{'table'});

   bless $self, $classname;
   return $self;
}


# destructor
sub dispose
{
   my ($self) = @_;
   $self->SUPER::dispose ();
   $self->{'dbh'}->disconnect;
}


# convert unix calendar time to ISO8601 format
sub toISO8601
{
   my ($self, $atime, $granularity) = @_;
   
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime ($atime);
   if ((defined $granularity) && ($granularity eq 'second'))
   {
      sprintf ("%04d-%02d-%02dT%02d:%02d:%02d+00:00", $year+1900, $mon+1, $mday, $hour, $min, $sec);
   }
   else
   {
      sprintf ("%04d-%02d-%02d", $year+1900, $mon+1, $mday);
   }
}


# convert an ISO8601 date into a unix calendar time representation
sub toSeconds
{
   my ($self, $date, $from) = @_;
   
   my ($hour, $minute, $second);
   
   if ((defined $from) && ($from == 1))
   {
      ($hour, $minute, $second) = (0, 0, 0);
   }
   else
   {
      ($hour, $minute, $second) = (23, 59, 59);
   }
   
   if ($date =~ /([0-9]{4})-([0-9]{2})-([0-9]{2})T([0-9]{2}):([0-9]{2}):([0-9]{2})[\+\-][0-9]{2}:[0-9]{2}/)
   {
      return mktime ($6, $5, $4, $3, $2-1, $1-1900);
   }
   elsif ($date =~ /([0-9]{4})-([0-9]{2})-([0-9]{2})/)
   {
      return mktime ($second, $minute, $hour, $3, $2-1, $1-1900);
   }
   return 0;
}

# component-specific identification
sub Archive_Identify
{
   my ($self) = @_;
   
   my $id = $self->SUPER::Archive_Identify ();
      
   if (! exists $id->{'description'})
   {
      $id->{'description'} = [];
   }
   my $desc = {
      'odl-description' => [[ 
         {
            'xmlns' => $self->{xoai_xmlnsprefix}.'/odl-description',
            'xsi:schemaLocation' => 
                       $self->{xoai_xmlnsprefix}.'/odl-description '.
                       $self->{xoai_xmlnsprefix}.'/odl-description.xsd'
         },
         {
            'protocol' => 'ODL-Union',
            'version'  => '1.0',
            'mdorder'  => [ qw ( protocol version ) ]
         } 
      ]]
   };
   push (@{$id->{'description'}}, $desc);
   
   $id;
}


# retrieve records from the source archive as required
sub Archive_FormatRecord
{
   my ($self, $hashref, $metadataFormat) = @_;
   
   if (defined $hashref->{$metadataFormat})
   {
      my $rowref = $hashref->{$metadataFormat};
      $self->FormatRecord ($rowref->{'identifier'},
                           $self->toISO8601 ($rowref->{'datestamp'}, 'second'),
                           $rowref->{'metadata'},
                           $rowref->{'about'},
                           $rowref->{'status'}
                          );
   }
   elsif ((keys %$hashref) == 0)
   {
      return '';
   }
   else
   {
      my $firstkey = (keys %$hashref)[0];
      my $rowref = $hashref->{$firstkey};
      $self->FormatRecord ($rowref->{'identifier'},
                           $self->toISO8601 ($rowref->{'datestamp'}, 'second'),
                           '',
                           '',
                           $rowref->{'status'}
                          );
   }
}


# get full list of mdps or list for specific identifier
sub Archive_ListMetadataFormats
{
   my ($self, $identifier) = @_;
   
   my %metadataNamespace = ();
   my %metadataSchema = ();
   my @mdps = ();
   
   # get full list of metadataPrefixes
   my $sth;
   if ($identifier ne '')
   {
      $sth = $self->{'dbh'}->prepare ("select metadataPrefix from $self->{'table'} ".
                                      "where identifier=\'$identifier\'");
   }
   else
   {
      $sth = $self->{'dbh'}->prepare ("select metadataPrefix from $self->{'table'}md");
   }
   $sth->execute;
   while (my $rowref = $sth->fetchrow_hashref)
   {
      push (@mdps, $rowref->{'metadataPrefix'});
   }
   $sth->finish;
   if ($#mdps == -1)
   {
      return [];
   }
   
   # get namespaces and schemas
   foreach my $mdp (@mdps)
   {
      $sth = $self->{'dbh'}->prepare ("select metadataNamespace,metadataSchema ".
                                      "from $self->{'table'}md ".
                                      "where metadataPrefix=\'$mdp\'");
      $sth->execute;
      if (my $rowref = $sth->fetchrow_hashref)
      {
         $metadataNamespace{$mdp} = $rowref->{'metadataNamespace'};
         $metadataSchema{$mdp} = $rowref->{'metadataSchema'};
      }
   }

   [ \%metadataNamespace, \%metadataSchema];
}


# generate list of sets from database
sub Archive_ListSets
{
   my ($self) = @_;
   
   my @setlist = ();
      
   my $sth = $self->{'dbh'}->prepare ("select distinct oaiset from $self->{'table'}");
   $sth->execute;
               
   while (my $rowref = $sth->fetchrow_hashref)
   {
      push (@setlist, [ $rowref->{oaiset}, $rowref->{oaiset} ]);
   }
                                 
   $sth->finish;
                                       
   \@setlist;      
}


# pass request through to source
sub Archive_GetRecord
{
   my ($self, $identifier) = @_;
   
   my %records = ();
      
   my $sth = $self->{'dbh'}->prepare ("select identifier,status,datestamp,metadataPrefix,".
                                      "metadata,about from $self->{'table'} ".
                                      "where identifier=\'$identifier\'");
   $sth->execute;
               
   while (my $rowref = $sth->fetchrow_hashref)
   {
      $records{$rowref->{'metadataPrefix'}} = {
         identifier => $rowref->{'identifier'},
         datestamp  => $rowref->{'datestamp'},
         status     => $rowref->{'status'},
         metadata   => $rowref->{'metadata'},
         about      => $rowref->{'about'}
      };
   }
   $sth->finish;

   \%records;
}


# return list of records/identifiers
sub Archive_ListRecords
{
   my ($self, $set, $from, $until, $metadataPrefix, $resumptionToken) = @_;

   my ($offset);
   if ($resumptionToken eq '')
   {
      $offset = 0;
   }
   else
   {
      my @rdata = split ($self->{resumptionseparator}, $resumptionToken);
      ($set, $from, $until, $metadataPrefix, $offset) = @rdata;
   }

   my $sth;

   # construct query
   my $sqlquery = "select";
   if ($metadataPrefix ne '') 
   { $sqlquery .= " * from $self->{'table'}"; }
   else
   { $sqlquery .= " distinct status,identifier from $self->{'table'}"; }

   if ($from eq '')
   {
      if ($until eq '') 
      { }
      else
      { $sqlquery = $sqlquery." where (datestamp <= \'".$self->toSeconds ($until)."\')"; }
   }
   else
   { 
      if ($until eq '')
      { $sqlquery = $sqlquery." where (datestamp >= \'".$self->toSeconds ($from, 1)."\')"; }
      else
      { $sqlquery = $sqlquery." where (datestamp >= \'".$self->toSeconds ($from, 1)."\') and (datestamp <= \'".$self->toSeconds ($until)."\')"; }
   }
   
   if ($metadataPrefix ne '')
   {
      if (($from eq '') && ($until eq ''))
      { $sqlquery .= ' where '; }
      else 
      { $sqlquery .= ' and '; }
      $sqlquery .= "(metadataPrefix = \'$metadataPrefix\')";
   }

   if ($set ne '')
   {
      if (($from eq '') && ($until eq '') && ($metadataPrefix eq ''))
      { $sqlquery .= ' where '; }
      else 
      { $sqlquery .= ' and '; }
      $sqlquery .= "(oaiset = \'$set\')";
   }   

   my $limit = $self->{recordlimit} + 1;
   $sqlquery = $sqlquery." limit $offset,$limit";
   $sth = $self->{'dbh'}->prepare ($sqlquery);
   $sth->execute;

   my @allrows = ();
   my $gotmore = 0;
   my $count = 0; 

   while (my $rowref = $sth->fetchrow_hashref)
   {
      if ($count == $self->{'recordlimit'})
      { $gotmore = 1; }
      else
      {
         $count++;
         if ($metadataPrefix eq '')
         {
            push (@allrows, { 
               urn    => $rowref->{'identifier'},
               status => $rowref->{'status'}
            });
         }
         else
         {
            push (@allrows, {
               $metadataPrefix => {
                  identifier => $rowref->{'identifier'},
                  datestamp  => $rowref->{'datestamp'},
                  status     => $rowref->{'status'},
                  metadata   => $rowref->{'metadata'},
                  about      => $rowref->{'about'}
               }
            });
         }
      }
   }

   $sth->finish;

   $resumptionToken = '';
   if ($gotmore == 1)
   {
      $resumptionToken = join ($self->{'resumptionseparator'}, ($set,$from,$until,$metadataPrefix,$offset+$self->{recordlimit}));
   }

   ( \@allrows, $resumptionToken, '', $metadataPrefix );
}


# forward request to ListRecords handler
sub Archive_ListIdentifiers
{
   my ($self, $set, $from, $until, $resumptionToken) = @_;

   $self->Archive_ListRecords ($set, $from, $until, '', $resumptionToken);
}


1;

