#  ---------------------------------------------------------------------
#   General database implementation of ODL-Union (v1.0) - service prov.  
#    v1.1
#    January 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package ODL::DBUnion::UnionSP;


use Pure::Log;
use Pure::X2D;
use Pure::EZXML;
use ODL::DBUnion::UnionDB;

use XOAI::Harvester;
use vars ('@ISA');
@ISA = ("XOAI::Harvester");


# constructor
sub new
{
   my ($classname, $configfile) = @_;

   my $self = $classname->SUPER::new ();

   # get configuration from file
   my $con = new Pure::X2D ($configfile);
   $self->{'database'} = $con->param ('database', '');
   $self->{'dbusername'} = $con->param ('dbusername', '');
   $self->{'dbpassword'} = $con->param ('dbpassword', '');
   $self->{'table'} = $con->param ('table', 'irdb');
   $self->{'log'} = new Pure::Log ('harvest.log');
   $self->{'counter'} = 0;
   $self->{'archive'} = $con->{'archive'};

   # open database
   $self->{'dbh'} = ODL::DBUnion::UnionDB::openDatabase ($self->{'database'},
     $self->{'dbusername'}, $self->{'dbpassword'}, $self->{'table'});
   
   bless $self, $classname;
   return $self;
}


# destructor
sub dispose
{
   my ($self) = @_;
   $self->{'dbh'}->disconnect;
}


# pass parameters and archive info to harvester
sub harvest
{
   my ($self, $when, $what, $fromwhere) = @_;
   $self->SUPER::harvest ($self->{'archive'}, $when, $what, $fromwhere);   
}


# output log file header
sub processHarvestStart
{
   my ($self, $archiveidentifier, $archiveurl, $archiveinterval, 
       $archivemetadataPrefixes, $archiveset, $archivedaysoverlap,
       $archiveinterrequestgap) = @_;
   $self->{'log'}->log ("");
   $self->{'log'}->log ("Harvesting:");
   $self->{'log'}->log ("  archive         = $archiveidentifier");
   $self->{'log'}->log ("  url             = $archiveurl");
   $self->{'log'}->log ("  interval        = $archiveinterval days");
   foreach my $archivemetadataPrefix (@$archivemetadataPrefixes)
   {
      $self->{'log'}->log ("  metadataPrefix  = $archivemetadataPrefix");
   }
   $self->{'log'}->log ("  set             = $archiveset");
   $self->{'log'}->log ("  daysoverlap     = $archivedaysoverlap");
   $self->{'log'}->log ("  interrequestgap = $archiveinterrequestgap");
   $self->{'log'}->log ("");
}


# end harvesting
sub processHarvestEnd
{
   my ($self) = @_;
}


# log the detection of an id
sub processIdStart
{
   my ($self, $identifier, $status) = @_;
   
   $self->{'counter'}++;
   $self->{'log'}->log ("[$self->{'counter'}] Processing : $identifier $status");
}

# end of records for an id
sub processIdEnd
{
   my ($self, $identifier) = @_;
}


# store a single record
sub processRecord
{
   my ($self, $xml, $date, $metadataPrefix, $set, $archiveid) = @_;
   
   # get identifier from header of record
   my $identifier = '';
   if ($xml->getElementsByTagName ("header")->getLength == 1)
   {
      my $header = $xml->getElementsByTagName ("header")->item(0);
      if ($header->getElementsByTagName ("identifier")->getLength == 1)
      {
         my $identifiercontainer = $header->getElementsByTagName ("identifier")->item(0);
         if (($identifiercontainer->hasChildNodes) && 
             ($identifiercontainer->getFirstChild->getNodeType == TEXT_NODE))
         {
            $identifier = $identifiercontainer->getFirstChild->getNodeValue;
         }
      }
   }
   $identifier =~ s/^([\s\r\n\t]*)([^\s\r\n\t]*)([\s\r\n\t]*)$/$2/;

   # get metadata chunk from record
   my $metadata = '';
   my $mdnamespace = '';
   my $mdschema = '';
   if ($xml->getElementsByTagName ("metadata")->getLength == 1)
   {
      my $metadatacontainer = $xml->getElementsByTagName ("metadata")->item(0);
      foreach my $metadatachunk ($metadatacontainer->getChildNodes)
      {
         if (($metadata eq '') && ($metadatachunk->getNodeType == ELEMENT_NODE))
         {
            $metadata = $metadatachunk->toString;
            my $mdschemaLocation = $metadatachunk->getAttribute ('schemaLocation');
            my @parts = split (' ', $mdschemaLocation);
            if ($#parts == 1)
            {
               $mdnamespace = $parts[0];
               $mdschema = $parts[1];
            }
         }
      }
   }

   # get about chunk from record
   my $about = '';
   if ($xml->getElementsByTagName ("about")->getLength == 1)
   {
      my $aboutcontainer = $xml->getElementsByTagName ("about")->item(0);
      foreach my $aboutchunk ($aboutcontainer->getChildNodes)
      {
         if (($about eq '') && ($aboutchunk->getNodeType == ELEMENT_NODE))
         {
            $about = $aboutchunk->toString;
         }
      }
   }

   # synthesize set from archive id and set   
   my $archiveset = $archiveid;
   if ($set ne '')
   {
      $archiveset .= '/'.$set;
   }

   my $datestamp = $self->toISO8601 ($date, 'second');
   
   my $status = $xml->getAttribute ('status');

   # for records that are ok ...
   if (($identifier ne '') && ($date ne '') && ($metadataPrefix ne '') && ($archiveid ne ''))
   {
      $self->{'log'}->log ("[$self->{'counter'}] Storing: $identifier, $datestamp, $metadataPrefix");
   
      # delete old record from table
      $self->{'dbh'}->do ("delete from $self->{'table'} where ".
                          "identifier=\'$identifier\' and ".
                          "metadataPrefix=\'$metadataPrefix\'");
      
      # insert new record into table
      $self->{'dbh'}->do ("insert into $self->{'table'} values (".
                           "\'$identifier\', ".
                           "\'$date\', ".
                           "\'$status\', ".
                           "\'$metadataPrefix\', ".
                           "\'$archiveset\', ".
                           $self->{'dbh'}->quote ($metadata).", ".
                           $self->{'dbh'}->quote ($about).
                           ")");
      
      # update schema information
      if (($mdnamespace ne '') && ($mdschema ne ''))
      {
         # insert new namespace and schema
         $self->{'dbh'}->do ("insert into $self->{'table'}md values (".
                             "\'$metadataPrefix\', ".
                             "\'$mdnamespace\', ".
                             "\'$mdschema\'".
                             ")");
      }
   }
}


1;

